/*
 * motion.c: Code for the motion kit driver SDK
 *
 * Copyright (C) 2007 Alexander Berl 'Raphael' <raphael@fx-world.org>
 *
 * This code is part of the motion kit driver SDK version 1.0
 */

#include <pspkernel.h>
#include <psploadcore.h>
#include <string.h>
#include "motion.h"

static SceUID load_module(const char *path, int flags, int type)
{
	SceKernelLMOption option;
	SceUID mpid;

	/* If the type is 0, then load the module in the kernel partition, otherwise load it
	   in the user partition. */
	if (type == 0) {
		mpid = 1;
	} else {
		mpid = 2;
	}

	memset(&option, 0, sizeof(option));
	option.size = sizeof(option);
	option.mpidtext = mpid;
	option.mpiddata = mpid;
	option.position = 0;
	option.access = 1;

	return sceKernelLoadModule(path, flags, type > 0 ? &option : NULL);
}

int motionLoad()
{
	SceUID modid = load_module("motion_driver.prx", 0, 0);

	// Module already loaded
	if (modid==0x80020139)
	{
		// Check if loaded plugin is an older version
		if (motionGetDriverVersion()<MOTION_SDK_VERSION && motionGetDriverVersion()>0x0100000b)
		{
			// motionUnload() only supported since 0x0100000c
			motionUnload();
			return motionLoad();
		}
		return 0;
	}
	
	if (modid>=0)
	{
		int fd;
		return sceKernelStartModule(modid, 0, NULL, &fd, NULL);
	}
	return modid;
}
